from django.core.management.base import BaseCommand

from rq import Queue, Worker
from redis.exceptions import ConnectionError

from django_rq import get_queue


def get_queues(*queue_names):
    """
    Return queue instances from specified queue names.
    All instances must use the same Redis connection.
    """
    from django_rq import settings
    if len(queue_names) > 1:
        connection_params = settings.QUEUES[queue_names[0]]
        for name in queue_names:
            if settings.QUEUES[name] != connection_params:
                raise ValueError('Queues in a single command must have the same '
                                 'redis connection. Queues "{0}" and "{1}" have '
                                 'different connections'.format(name, queue_names[0]))
    return [get_queue(name) for name in queue_names]


class Command(BaseCommand):
    """
    Runs RQ workers on specified queues. Note that all queues passed into a single rqworker
    command must share the same connection.

    Example usage:
    python manage.py rqworker high medium low
    """
    args = '<queue queue ...>'

    def handle(self, *args, **options):        
        if args:
            queues = get_queues(*args)
        else:
            from django_rq import settings
            queues = get_queues(*settings.QUEUES.keys())
        try:
            w = Worker(queues, connection=queues[0].connection)
            w.work()
        except ConnectionError as e:
            print(e)



from django.core.exceptions import ImproperlyConfigured
from django.core.management import call_command
from django.test import TestCase
from django.utils.unittest import skipIf
from django.test.utils import override_settings

from rq.job import Job

from .management.commands.rqworker import get_queues
from .queues import get_connection, get_queue

try:
    from rq_scheduler import Scheduler
    from .queues import get_scheduler
    RQ_SCHEDULER_INSTALLED = True
except ImportError:
    RQ_SCHEDULER_INSTALLED = False


TEST_QUEUES = {
    'default': {
        'HOST': 'localhost',
        'PORT': 6379,
        'DB': 0,
    },
    'test': {
        'HOST': 'localhost',
        'PORT': 1,
        'DB': 1,
    },
    'test2': {
        'HOST': 'localhost',
        'PORT': 1,
        'DB': 1,
    },
}


class DjangoRQTest(TestCase):

    @override_settings(RQ_QUEUES=TEST_QUEUES)
    def test_get_connection(self):
        """
        Test that get_connection returns the right connection based on 
        settings.RQ_QUEUES
        """
        config = TEST_QUEUES['default']
        connection = get_connection()
        connection_kwargs = connection.connection_pool.connection_kwargs
        self.assertEqual(connection_kwargs['host'], config['HOST'])
        self.assertEqual(connection_kwargs['port'], config['PORT'])
        self.assertEqual(connection_kwargs['db'], config['DB'])

        config = TEST_QUEUES['test']
        connection = get_connection('test')
        connection_kwargs = connection.connection_pool.connection_kwargs
        self.assertEqual(connection_kwargs['host'], config['HOST'])
        self.assertEqual(connection_kwargs['port'], config['PORT'])
        self.assertEqual(connection_kwargs['db'], config['DB'])

    @override_settings(RQ_QUEUES=TEST_QUEUES)
    def test_get_queue(self):
        # Test that get_queue use the right parameters for its connection
        config = TEST_QUEUES['default']
        queue = get_queue('default')
        connection_kwargs = queue.connection.connection_pool.connection_kwargs
        self.assertEqual(queue.name, 'default')
        self.assertEqual(connection_kwargs['host'], config['HOST'])
        self.assertEqual(connection_kwargs['port'], config['PORT'])
        self.assertEqual(connection_kwargs['db'], config['DB'])

        config = TEST_QUEUES['test']
        queue = get_queue('test')
        connection_kwargs = queue.connection.connection_pool.connection_kwargs
        self.assertEqual(queue.name, 'test')
        self.assertEqual(connection_kwargs['host'], config['HOST'])
        self.assertEqual(connection_kwargs['port'], config['PORT'])
        self.assertEqual(connection_kwargs['db'], config['DB'])

    @override_settings(RQ_QUEUES=None)
    def test_empty_queue_setting_raises_exception(self):
        # Raise an exception if RQ_QUEUES is not defined
        self.assertRaises(ImproperlyConfigured, get_connection)

    @override_settings(RQ_QUEUES=TEST_QUEUES)
    def test_get_queues(self):        
        # Getting queues with the same redis connection is ok
        self.assertEqual(get_queues('test', 'test2'), [get_queue('test'), get_queue('test2')])
        # Getting queues with different connections raises an exception
        self.assertRaises(ValueError, get_queues, 'default', 'test')

    @override_settings(RQ_QUEUES=TEST_QUEUES)
    @skipIf(RQ_SCHEDULER_INSTALLED is False, 'RQ Scheduler not installed')
    def test_get_scheduler(self):        
        """
        Ensure get_scheduler creates a scheduler instance with the right
        connection params.
        """
        
        config = TEST_QUEUES['default']
        scheduler = get_scheduler()
        connection_kwargs = scheduler.connection.connection_pool.connection_kwargs
        self.assertEqual(scheduler.queue_name, 'default')
        self.assertEqual(connection_kwargs['host'], config['HOST'])
        self.assertEqual(connection_kwargs['port'], config['PORT'])
        self.assertEqual(connection_kwargs['db'], config['DB'])        

        config = TEST_QUEUES['test']
        scheduler = get_scheduler('test')
        connection_kwargs = scheduler.connection.connection_pool.connection_kwargs
        self.assertEqual(scheduler.queue_name, 'test')
        self.assertEqual(connection_kwargs['host'], config['HOST'])
        self.assertEqual(connection_kwargs['port'], config['PORT'])
        self.assertEqual(connection_kwargs['db'], config['DB'])        

